@@ -73,7 +73,7 @@ public:
     //! Check if we consider the last connect as reconnect
     bool wasReconnect() const { return _wasReconnect; }
 
-    QPointer<Peer> peer() { return _peer; }
+    QPointer<Peer> peer() const;
 
 public slots:
     bool connectToCore(AccountId = 0);
@@ -97,12 +97,11 @@ signals:
     void progressValueChanged(int value);
     void progressTextChanged(const QString &);
 
-    void startCoreSetup(const QVariantList &backendInfo);
+    void startCoreSetup(const QVariantList &backendInfo, const QVariantList &authenticatorInfo);
     void coreSetupSuccess();
     void coreSetupFailed(const QString &error);
 
-    void startInternalCore();
-    void connectToInternalCore(InternalPeer *connection);
+    void connectToInternalCore(QPointer<InternalPeer> connection);
 
     // These signals MUST be handled synchronously!
     void userAuthenticationRequired(CoreAccount *, bool *valid, const QString &errorMessage = QString());
