@@ -47,10 +47,22 @@ BufferViewConfig *BufferViewManager::bufferViewConfigFactory(int bufferViewConfi
 }
 
 
+void BufferViewManager::addBufferViewConfig(int bufferViewConfigId)
+{
+    if (_bufferViewConfigs.contains(bufferViewConfigId)) {
+        return;
+    }
+
+    addBufferViewConfig(bufferViewConfigFactory(bufferViewConfigId));
+}
+
+
 void BufferViewManager::addBufferViewConfig(BufferViewConfig *config)
 {
-    if (_bufferViewConfigs.contains(config->bufferViewId()))
+    if (_bufferViewConfigs.contains(config->bufferViewId())) {
+        delete config;
         return;
+    }
 
     _proxy->synchronize(config);
     _bufferViewConfigs[config->bufferViewId()] = config;
@@ -60,15 +72,6 @@ void BufferViewManager::addBufferViewConfig(BufferViewConfig *config)
 }
 
 
-void BufferViewManager::addBufferViewConfig(int bufferViewConfigId)
-{
-    if (_bufferViewConfigs.contains(bufferViewConfigId))
-        return;
-
-    addBufferViewConfig(bufferViewConfigFactory(bufferViewConfigId));
-}
-
-
 void BufferViewManager::deleteBufferViewConfig(int bufferViewConfigId)
 {
     if (!_bufferViewConfigs.contains(bufferViewConfigId))
@@ -96,10 +99,7 @@ QVariantList BufferViewManager::initBufferViewIds() const
 
 void BufferViewManager::initSetBufferViewIds(const QVariantList bufferViewIds)
 {
-    QVariantList::const_iterator iter = bufferViewIds.constBegin();
-    QVariantList::const_iterator iterEnd = bufferViewIds.constEnd();
-    while (iter != iterEnd) {
-        newBufferViewConfig((*iter).value<int>());
-        ++iter;
+    for (auto &&id : bufferViewIds) {
+        addBufferViewConfig(id.value<int>());
     }
 }
