@@ -53,8 +53,50 @@ public:
     //! Sets up notification and calls the given slot to set the initial value
     void initAndNotify(const QString &key, QObject *receiver, const char *slot, const QVariant &defaultValue = QVariant());
 
+    /**
+     * Get the major configuration version
+     *
+     * This indicates the backwards/forwards incompatible version of configuration.
+     *
+     * @return Major configuration version (the X in XX.YY)
+     */
     virtual uint version();
 
+    /**
+     * Get the minor configuration version
+     *
+     * This indicates the backwards/forwards compatible version of configuration.
+     *
+     * @see Settings::setVersionMinor()
+     * @return Minor configuration version (the Y in XX.YY)
+     */
+    virtual uint versionMinor();
+
+    /**
+     * Set the minor configuration version
+     *
+     * When making backwards/forwards compatible changes, call this with the new version number.
+     * This does not implement any upgrade logic; implement that when checking Settings::version(),
+     * e.g. in Core::Core() and QtUiApplication::init().
+     *
+     * @param[in] versionMinor New minor version number
+     */
+    virtual void setVersionMinor(const uint versionMinor);
+
+    /**
+     * Persist unsaved changes to permanent storage
+     *
+     * @return true if succeeded, false otherwise
+     */
+    bool sync();
+
+    /**
+     * Check if the configuration storage is writable.
+     *
+     * @return true if writable, false otherwise
+     */
+    bool isWritable();
+
 protected:
     inline Settings(QString group_, QString appName_) : group(group_), appName(appName_) {}
     inline virtual ~Settings() {}
@@ -66,7 +108,15 @@ protected:
     virtual QStringList localChildGroups(const QString &rootkey = QString());
 
     virtual void setLocalValue(const QString &key, const QVariant &data);
-    virtual const QVariant &localValue(const QString &key, const QVariant &def = QVariant());
+    virtual QVariant localValue(const QString &key, const QVariant &def = QVariant());
+
+    /**
+     * Gets if a key exists in settings
+     *
+     * @param[in] key ID of local settings key
+     * @returns True if key exists in settings, otherwise false
+     */
+    virtual bool localKeyExists(const QString &key);
 
     virtual void removeLocalKey(const QString &key);
 
@@ -91,7 +141,8 @@ private:
     }
 
 
-    static QHash<QString, QVariant> settingsCache;
+    static QHash<QString, QVariant> settingsCache;         ///< Cached settings values
+    static QHash<QString, bool> settingsKeyPersistedCache; ///< Cached settings key exists on disk
     static QHash<QString, SettingsChangeNotifier *> settingsChangeNotifier;
 
     inline QString normalizedKey(const QString &group, const QString &key)
@@ -102,6 +153,44 @@ private:
     }
 
 
+    /**
+     * Update the cache of whether or not a given settings key persists on disk
+     *
+     * @param normKey Normalized settings key ID
+     * @param exists  True if key exists, otherwise false
+     */
+    inline void setCacheKeyPersisted(const QString &normKey, bool exists)
+    {
+        settingsKeyPersistedCache[normKey] = exists;
+    }
+
+
+    /**
+     * Check if the given settings key ID persists on disk (rather than being a default value)
+     *
+     * @see Settings::localKeyExists()
+     *
+     * @param normKey Normalized settings key ID
+     * @return True if key exists and persistence has been cached, otherwise false
+     */
+    inline const bool &cacheKeyPersisted(const QString &normKey)
+    {
+        return settingsKeyPersistedCache[normKey];
+    }
+
+
+    /**
+     * Check if the persistence of the given settings key ID has been cached
+     *
+     * @param normKey Normalized settings key ID
+     * @return True if key persistence has been cached, otherwise false
+     */
+    inline bool isKeyPersistedCached(const QString &normKey)
+    {
+        return settingsKeyPersistedCache.contains(normKey);
+    }
+
+
     inline void setCacheValue(const QString &normKey, const QVariant &data)
     {
         settingsCache[normKey] = data;
