@@ -156,9 +156,11 @@ int SignalProxy::SignalRelay::qt_metacall(QMetaObject::Call _c, int _id, void **
                 params << QVariant(argTypes[i], _a[i+1]);
             }
 
-            if (argTypes.size() >= 1 && argTypes[0] == qMetaTypeId<PeerPtr>() && proxy()->proxyMode() == SignalProxy::Server) {
-                Peer *peer = params[0].value<PeerPtr>();
-                proxy()->dispatch(peer, RpcCall(signal.signature, params));
+            if (proxy()->_restrictMessageTarget) {
+                for (auto peer : proxy()->_restrictedTargets) {
+                    if (peer != nullptr)
+                        proxy()->dispatch(peer, RpcCall(signal.signature, params));
+                }
             } else
                 proxy()->dispatch(RpcCall(signal.signature, params));
         }
@@ -171,6 +173,9 @@ int SignalProxy::SignalRelay::qt_metacall(QMetaObject::Call _c, int _id, void **
 // ==================================================
 //  SignalProxy
 // ==================================================
+
+thread_local SignalProxy *SignalProxy::_current{nullptr};
+
 SignalProxy::SignalProxy(QObject *parent)
     : QObject(parent)
 {
@@ -202,12 +207,14 @@ SignalProxy::~SignalProxy()
     _syncSlave.clear();
 
     removeAllPeers();
+
+    _current = nullptr;
 }
 
 
 void SignalProxy::setProxyMode(ProxyMode mode)
 {
-    if (_peers.count()) {
+    if (!_peerMap.empty()) {
         qWarning() << Q_FUNC_INFO << "Cannot change proxy mode while connected";
         return;
     }
@@ -219,7 +226,6 @@ void SignalProxy::setProxyMode(ProxyMode mode)
         initClient();
 }
 
-
 void SignalProxy::init()
 {
     _heartBeatInterval = 0;
@@ -228,6 +234,7 @@ void SignalProxy::init()
     setHeartBeatInterval(30);
     setMaxHeartBeatCount(2);
     _secure = false;
+    _current = this;
     updateSecureState();
 }
 
@@ -266,7 +273,7 @@ bool SignalProxy::addPeer(Peer *peer)
     if (!peer)
         return false;
 
-    if (_peers.contains(peer))
+    if (_peerMap.values().contains(peer))
         return true;
 
     if (!peer->isOpen()) {
@@ -275,7 +282,7 @@ bool SignalProxy::addPeer(Peer *peer)
     }
 
     if (proxyMode() == Client) {
-        if (!_peers.isEmpty()) {
+        if (!_peerMap.isEmpty()) {
             qWarning("SignalProxy: only one peer allowed in client mode!");
             return false;
         }
@@ -288,11 +295,16 @@ bool SignalProxy::addPeer(Peer *peer)
     if (!peer->parent())
         peer->setParent(this);
 
-    _peers.insert(peer);
+    if (peer->id() < 0) {
+        peer->setId(nextPeerId());
+        peer->setConnectedSince(QDateTime::currentDateTimeUtc());
+    }
+
+    _peerMap[peer->id()] = peer;
 
     peer->setSignalProxy(this);
 
-    if (_peers.count() == 1)
+    if (peerCount() == 1)
         emit connected();
 
     updateSecureState();
@@ -302,10 +314,10 @@ bool SignalProxy::addPeer(Peer *peer)
 
 void SignalProxy::removeAllPeers()
 {
-    Q_ASSERT(proxyMode() == Server || _peers.count() <= 1);
+    Q_ASSERT(proxyMode() == Server || peerCount() <= 1);
     // wee need to copy that list since we modify it in the loop
-    QSet<Peer *> peers = _peers;
-    foreach(Peer *peer, peers) {
+    QList<Peer *> peers = _peerMap.values();
+    for (auto peer : peers) {
         removePeer(peer);
     }
 }
@@ -318,12 +330,12 @@ void SignalProxy::removePeer(Peer *peer)
         return;
     }
 
-    if (_peers.isEmpty()) {
+    if (_peerMap.isEmpty()) {
         qWarning() << "SignalProxy::removePeer(): No peers in use!";
         return;
     }
 
-    if (!_peers.contains(peer)) {
+    if (!_peerMap.values().contains(peer)) {
         qWarning() << "SignalProxy: unknown Peer" << peer;
         return;
     }
@@ -331,7 +343,7 @@ void SignalProxy::removePeer(Peer *peer)
     disconnect(peer, 0, this, 0);
     peer->setSignalProxy(0);
 
-    _peers.remove(peer);
+    _peerMap.remove(peer->id());
     emit peerRemoved(peer);
 
     if (peer->parent() == this)
@@ -339,7 +351,7 @@ void SignalProxy::removePeer(Peer *peer)
 
     updateSecureState();
 
-    if (_peers.isEmpty())
+    if (_peerMap.isEmpty())
         emit disconnected();
 }
 
@@ -463,8 +475,11 @@ void SignalProxy::synchronize(SyncableObject *obj)
 
 void SignalProxy::detachObject(QObject *obj)
 {
-    detachSignals(obj);
-    detachSlots(obj);
+    // Don't try to connect SignalProxy from itself on shutdown
+    if (obj != this) {
+        detachSignals(obj);
+        detachSlots(obj);
+    }
 }
 
 
@@ -506,11 +521,8 @@ void SignalProxy::stopSynchronize(SyncableObject *obj)
 template<class T>
 void SignalProxy::dispatch(const T &protoMessage)
 {
-    foreach (Peer *peer, _peers) {
-        if (peer->isOpen())
-            peer->dispatch(protoMessage);
-        else
-            QCoreApplication::postEvent(this, new ::RemovePeerEvent(peer));
+    for (auto&& peer : _peerMap.values()) {
+        dispatch(peer, protoMessage);
     }
 }
 
@@ -518,10 +530,14 @@ void SignalProxy::dispatch(const T &protoMessage)
 template<class T>
 void SignalProxy::dispatch(Peer *peer, const T &protoMessage)
 {
+    _targetPeer = peer;
+
     if (peer && peer->isOpen())
         peer->dispatch(protoMessage);
     else
         QCoreApplication::postEvent(this, new ::RemovePeerEvent(peer));
+
+    _targetPeer = nullptr;
 }
 
 
@@ -564,7 +580,9 @@ void SignalProxy::handle(Peer *peer, const SyncMessage &syncMessage)
         if (eMeta->argTypes(receiverId).count() > 1)
             returnParams << syncMessage.params;
         returnParams << returnValue;
+        _targetPeer = peer;
         peer->dispatch(SyncMessage(syncMessage.className, syncMessage.objectName, eMeta->methodName(receiverId), returnParams));
+        _targetPeer = nullptr;
     }
 
     // send emit update signal
@@ -587,7 +605,9 @@ void SignalProxy::handle(Peer *peer, const InitRequest &initRequest)
     }
 
     SyncableObject *obj = _syncSlave[initRequest.className][initRequest.objectName];
+    _targetPeer = peer;
     peer->dispatch(InitData(initRequest.className, initRequest.objectName, initData(obj)));
+    _targetPeer = nullptr;
 }
 
 
@@ -661,12 +681,8 @@ bool SignalProxy::invokeSlot(QObject *receiver, int methodId, const QVariantList
             qWarning() << "SignalProxy::invokeSlot(): incompatible param types to invoke" << eMeta->methodName(methodId);
             return false;
         }
-        // if first arg is a PeerPtr, replace it by the address of the peer originally receiving the RpcCall
-        if (peer && i == 0 && args[0] == qMetaTypeId<PeerPtr>()) {
-            QVariant v = QVariant::fromValue<PeerPtr>(peer);
-            _a[1] = const_cast<void*>(v.constData());
-        } else
-            _a[i+1] = const_cast<void *>(params[i].constData());
+
+        _a[i+1] = const_cast<void *>(params[i].constData());
     }
 
     if (returnValue.type() != QVariant::Invalid)
@@ -677,9 +693,11 @@ bool SignalProxy::invokeSlot(QObject *receiver, int methodId, const QVariantList
                               : Qt::QueuedConnection;
 
     if (type == Qt::DirectConnection) {
-        return receiver->qt_metacall(QMetaObject::InvokeMetaMethod, methodId, _a) < 0;
-    }
-    else {
+        _sourcePeer = peer;
+        auto result = receiver->qt_metacall(QMetaObject::InvokeMetaMethod, methodId, _a) < 0;
+        _sourcePeer = nullptr;
+        return result;
+    } else {
         qWarning() << "Queued Connections are not implemented yet";
         // note to self: qmetaobject.cpp:990 ff
         return false;
@@ -758,9 +776,11 @@ void SignalProxy::sync_call__(const SyncableObject *obj, SignalProxy::ProxyMode
         params << QVariant(argTypes[i], va_arg(ap, void *));
     }
 
-    if (argTypes.size() >= 1 && argTypes[0] == qMetaTypeId<PeerPtr>() && proxyMode() == SignalProxy::Server) {
-        Peer *peer = params[0].value<PeerPtr>();
-        dispatch(peer, SyncMessage(eMeta->metaObject()->className(), obj->objectName(), QByteArray(funcname), params));
+    if (_restrictMessageTarget) {
+        for (auto peer : _restrictedTargets) {
+            if (peer != nullptr)
+                dispatch(peer, SyncMessage(eMeta->metaObject()->className(), obj->objectName(), QByteArray(funcname), params));
+        }
     } else
         dispatch(SyncMessage(eMeta->metaObject()->className(), obj->objectName(), QByteArray(funcname), params));
 }
@@ -801,8 +821,8 @@ void SignalProxy::updateSecureState()
 {
     bool wasSecure = _secure;
 
-    _secure = !_peers.isEmpty();
-    foreach (const Peer *peer,  _peers) {
+    _secure = !_peerMap.isEmpty();
+    for (auto peer :  _peerMap.values()) {
         _secure &= peer->isSecure();
     }
 
@@ -810,6 +830,60 @@ void SignalProxy::updateSecureState()
         emit secureStateChanged(_secure);
 }
 
+QVariantList SignalProxy::peerData() {
+    QVariantList result;
+    for (auto &&peer : _peerMap.values()) {
+        QVariantMap data;
+        data["id"] = peer->id();
+        data["clientVersion"] = peer->clientVersion();
+        // We explicitly rename this, as, due to the Debian reproducability changes, buildDate isn’t actually the build
+        // date anymore, but on newer clients the date of the last git commit
+        data["clientVersionDate"] = peer->buildDate();
+        data["remoteAddress"] = peer->address();
+        data["connectedSince"] = peer->connectedSince();
+        data["secure"] = peer->isSecure();
+        data["features"] = static_cast<quint32>(peer->features().toLegacyFeatures());
+        data["featureList"] = peer->features().toStringList();
+        result << data;
+    }
+    return result;
+}
+
+Peer *SignalProxy::peerById(int peerId) {
+    // We use ::value() here instead of the [] operator because the latter has the side-effect
+    // of automatically inserting a null value with the passed key into the map.  See
+    // https://doc.qt.io/qt-5/qhash.html#operator-5b-5d and https://doc.qt.io/qt-5/qhash.html#value.
+    return _peerMap.value(peerId);
+}
+
+void SignalProxy::restrictTargetPeers(QSet<Peer*> peers, std::function<void()> closure)
+{
+    auto previousRestrictMessageTarget = _restrictMessageTarget;
+    auto previousRestrictedTargets = _restrictedTargets;
+    _restrictMessageTarget = true;
+    _restrictedTargets = peers;
+
+    closure();
+
+    _restrictMessageTarget = previousRestrictMessageTarget;
+    _restrictedTargets = previousRestrictedTargets;
+}
+
+Peer *SignalProxy::sourcePeer() {
+    return _sourcePeer;
+}
+
+void SignalProxy::setSourcePeer(Peer *sourcePeer) {
+    _sourcePeer = sourcePeer;
+}
+
+Peer *SignalProxy::targetPeer() {
+    return _targetPeer;
+}
+
+void SignalProxy::setTargetPeer(Peer *targetPeer) {
+    _targetPeer = targetPeer;
+}
 
 // ==================================================
 //  ExtendedMetaObject
