@@ -18,8 +18,7 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-#ifndef CHATMONITORFILTER_H_
-#define CHATMONITORFILTER_H_
+#pragma once
 
 #include <QDateTime>
 
@@ -55,21 +54,27 @@ public slots:
 private slots:
     void showFieldsSettingChanged(const QVariant &newValue);
     void showOwnMessagesSettingChanged(const QVariant &newValue);
+    void alwaysOwnSettingChanged(const QVariant &newValue);
     void showHighlightsSettingChanged(const QVariant &newValue);
     void operationModeSettingChanged(const QVariant &newValue);
     void buffersSettingChanged(const QVariant &newValue);
     void showBacklogSettingChanged(const QVariant &newValue);
     void includeReadSettingChanged(const QVariant &newValue);
+    /**
+     * Updates the local setting cache of whether or not to show sender brackets
+     *
+     * @param[in] newValue  If true, sender brackets are enabled, otherwise false.
+     */
+    void showSenderBracketsSettingChanged(const QVariant &newValue);
 
 private:
     int _showFields;
     bool _showOwnMessages;
+    bool _alwaysOwn;
     QList<BufferId> _bufferIds;
     bool _showHighlights;
     int _operationMode;
     bool _showBacklog;
     bool _includeRead;
+    bool _showSenderBrackets;   /// If true, show brackets around sender names
 };
-
-
-#endif
