@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2018 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,6 +20,8 @@
 
 #pragma once
 
+#include <QUuid>
+
 #ifdef HAVE_KDE4
 #  include <KMainWindow>
 #elif defined HAVE_KF5
@@ -36,8 +38,8 @@ class ActionCollection;
 class BufferHotListFilter;
 class BufferView;
 class BufferViewConfig;
+class ChatMonitorView;
 class ClientBufferViewConfig;
-class ClientTransfer;
 class CoreAccount;
 class CoreConnectionStatusWidget;
 class BufferViewDock;
@@ -46,11 +48,11 @@ class InputWidget;
 class MsgProcessorStatusWidget;
 class NickListWidget;
 class SystemTray;
-class ChatMonitorView;
 class TopicWidget;
 
-class QMenu;
 class QLabel;
+class QMenu;
+class QMessageBox;
 class QToolBar;
 
 class KHelpMenu;
@@ -66,7 +68,6 @@ class MainWin
 
 public:
     MainWin(QWidget *parent = 0);
-    virtual ~MainWin();
 
     void init();
 
@@ -85,6 +86,9 @@ public:
     void saveStateToSettings(UiSettings &);
     void restoreStateFromSettings(UiSettings &);
 
+    // We need to override this to add the show/hide menu bar option
+    virtual QMenu *createPopupMenu();
+
 public slots:
     void showStatusBarMessage(const QString &message);
     void hideCurrentBuffer();
@@ -93,8 +97,9 @@ public slots:
     void nextBuffer();
     void previousBuffer();
 
-    //! Quit application
-    void quit();
+    void showMigrationWarning(bool show);
+
+    void onExitRequested(const QString &reason);
 
 protected:
     void closeEvent(QCloseEvent *event);
@@ -114,9 +119,20 @@ private slots:
     void currentBufferChanged(BufferId);
     void messagesInserted(const QModelIndex &parent, int start, int end);
     void showAboutDlg();
-    void showChannelList(NetworkId netId = NetworkId());
+
+    /**
+     * Show the channel list dialog for the network, optionally searching by channel name
+     *
+     * @param networkId        Network ID for associated network
+     * @param channelFilters   Partial channel name to search for, or empty to show all
+     * @param listImmediately  If true, immediately list channels, otherwise just show dialog
+     */
+    void showChannelList(NetworkId netId = {}, const QString &channelFilters = {},
+                         bool listImmediately = false);
+
+    void showNetworkConfig(NetworkId netId = NetworkId());
     void showCoreConnectionDlg();
-    void showCoreConfigWizard(const QVariantList &);
+    void showCoreConfigWizard(const QVariantList &, const QVariantList &);
     void showCoreInfoDlg();
     void showAwayLog();
     void showSettingsDlg();
@@ -124,9 +140,11 @@ private slots:
     void showIgnoreList(QString newRule = QString());
     void showShortcutsDlg();
     void showPasswordChangeDlg();
-    void showNewTransferDlg(const ClientTransfer *transfer);
+    void showNewTransferDlg(const QUuid &transferId);
     void onFullScreenToggled();
 
+    void doAutoConnect();
+
     void handleCoreConnectionError(const QString &errorMsg);
     void userAuthenticationRequired(CoreAccount *, bool *valid, const QString &errorMessage);
     void handleNoSslInClient(bool *accepted);
@@ -138,7 +156,51 @@ private slots:
     void on_actionConfigureNetworks_triggered();
     void on_actionConfigureViews_triggered();
     void on_actionLockLayout_toggled(bool lock);
+
+    /**
+     * Apply the active color to the input widget selected or typed text
+     *
+     * @seealso InputWidget::applyFormatActiveColor()
+     */
+    void on_inputFormatApplyColor_triggered();
+
+    /**
+     * Apply the active fill color to the input widget selected or typed text background
+     *
+     * @seealso InputWidget::applyFormatActiveColorFill()
+     */
+    void on_inputFormatApplyColorFill_triggered();
+
+    /**
+     * Toggle the boldness of the input widget selected or typed text
+     *
+     * @seealso InputWidget::toggleFormatBold()
+     */
+    void on_inputFormatBold_triggered();
+
+    /**
+     * Toggle the italicness of the input widget selected or typed text
+     *
+     * @seealso InputWidget::toggleFormatItalic()
+     */
+    void on_inputFormatItalic_triggered();
+
+    /**
+     * Toggle the underlining of the input widget selected or typed text
+     *
+     * @seealso InputWidget::toggleFormatUnderline()
+     */
+    void on_inputFormatUnderline_triggered();
+
+    /**
+     * Clear the formatting of the input widget selected or typed text
+     *
+     * @seealso InputWidget::clearFormat()
+     */
+    void on_inputFormatClear_triggered();
+
     void on_jumpHotBuffer_triggered();
+    void on_bufferSearch_triggered();
     void on_actionDebugNetworkModel_triggered();
     void on_actionDebugBufferViewOverlay_triggered();
     void on_actionDebugMessageModel_triggered();
@@ -187,6 +249,7 @@ private:
     void setupChatMonitor();
     void setupInputWidget();
     void setupTopicWidget();
+    void setupTransferWidget();
     void setupViewMenuTail();
     void setupStatusBar();
     void setupSystray();
@@ -211,6 +274,8 @@ private:
 
     QWidget *_awayLog;
 
+    QMessageBox* _migrationWarning{nullptr};
+
     bool _layoutLoaded;
 
     QSize _normalSize; //!< Size of the non-maximized window
@@ -220,5 +285,7 @@ private:
     QHash<int, BufferId> _jumpKeyMap;
     int _activeBufferViewIndex;
 
+    bool _aboutToQuit; //closeEvent can occur multiple times on OSX
+
     friend class QtUi;
 };
