@@ -18,38 +18,40 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-#ifndef MONOAPPLICATION_H_
-#define MONOAPPLICATION_H_
+#pragma once
+
+#include <QPointer>
+#include <QThread>
 
 #include "qtuiapplication.h"
 
-class CoreApplicationInternal;
+class Core;
+class InternalPeer;
 
 class MonolithicApplication : public QtUiApplication
 {
     Q_OBJECT
+
 public:
     MonolithicApplication(int &, char **);
-    ~MonolithicApplication();
 
-    bool init();
+    void init() override;
 
-    /**
-     * Requests a reload of relevant runtime configuration.
-     *
-     * @see Quassel::reloadConfig()
-     *
-     * @return True if configuration reload successful, otherwise false
-     */
-    bool reloadConfig();
+protected:
+    Quassel::QuitHandler quitHandler() override;
+
+signals:
+    void connectInternalPeer(QPointer<InternalPeer> peer);
 
 private slots:
+    void onConnectionRequest(QPointer<InternalPeer> peer);
+    void onClientDestroyed();
+    void onCoreShutdown();
+
+private:
     void startInternalCore();
 
 private:
-    CoreApplicationInternal *_internal;
-    bool _internalInitDone;
+    QPointer<Core> _core;
+    QThread _coreThread;
 };
-
-
-#endif
