@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2018 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -53,7 +53,9 @@ private:
     bool testHasChanged();
     void initStyleComboBox();
     void initLanguageComboBox();
+    void initIconThemeComboBox();
     QLocale selectedLocale() const;
+    QString selectedIconTheme() const;
 
     Ui::AppearanceSettingsPage ui;
     QHash<QString, QVariant> settings;
