@@ -20,11 +20,11 @@
 
 #include "identitiessettingspage.h"
 
-#include <QIcon>
 #include <QInputDialog>
 #include <QMessageBox>
 
 #include "client.h"
+#include "icon.h"
 #include "signalproxy.h"
 
 IdentitiesSettingsPage::IdentitiesSettingsPage(QWidget *parent)
@@ -32,9 +32,9 @@ IdentitiesSettingsPage::IdentitiesSettingsPage(QWidget *parent)
     _editSsl(false)
 {
     ui.setupUi(this);
-    ui.renameIdentity->setIcon(QIcon::fromTheme("edit-rename"));
-    ui.addIdentity->setIcon(QIcon::fromTheme("list-add-user"));
-    ui.deleteIdentity->setIcon(QIcon::fromTheme("list-remove-user"));
+    ui.renameIdentity->setIcon(icon::get("edit-rename"));
+    ui.addIdentity->setIcon(icon::get("list-add-user"));
+    ui.deleteIdentity->setIcon(icon::get("list-remove-user"));
 
     coreConnectionStateChanged(Client::isConnected()); // need a core connection!
     connect(Client::instance(), SIGNAL(coreConnectionStateChanged(bool)), this, SLOT(coreConnectionStateChanged(bool)));
@@ -417,7 +417,7 @@ SaveIdentitiesDlg::SaveIdentitiesDlg(const QList<CertIdentity *> &toCreate, cons
     : QDialog(parent)
 {
     ui.setupUi(this);
-    ui.abort->setIcon(QIcon::fromTheme("dialog-cancel"));
+    ui.abort->setIcon(icon::get("dialog-cancel"));
 
     numevents = toCreate.count() + toUpdate.count() + toRemove.count();
     rcvevents = 0;
