@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2015 by the Quassel Project                        *
+ *   Copyright (C) 2005-2018 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -28,6 +28,7 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QSet>
+#include <QVBoxLayout>
 
 #include "action.h"
 #include "buffermodel.h"
@@ -45,9 +46,9 @@
 * The TreeView showing the Buffers
 *****************************************/
 // Please be carefull when reimplementing methods which are used to inform the view about changes to the data
-// to be on the safe side: call QTreeView's method aswell
+// to be on the safe side: call QTreeView's method aswell (or TreeViewTouch's)
 BufferView::BufferView(QWidget *parent)
-    : QTreeView(parent)
+    : TreeViewTouch(parent)
 {
     connect(this, SIGNAL(collapsed(const QModelIndex &)), SLOT(storeExpandedState(const QModelIndex &)));
     connect(this, SIGNAL(expanded(const QModelIndex &)), SLOT(storeExpandedState(const QModelIndex &)));
@@ -68,10 +69,12 @@ void BufferView::init()
     hideColumn(2);
     setIndentation(10);
 
-    expandAll();
+    // New entries will be expanded automatically when added; no need to call expandAll()
 
     header()->hide(); // nobody seems to use this anyway
 
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+
     // breaks with Qt 4.8
     if (QString("4.8.0") > qVersion()) // FIXME breaks with Qt versions >= 4.10!
         setAnimated(true);
@@ -88,14 +91,13 @@ void BufferView::init()
     setSortingEnabled(true);
     sortByColumn(0, Qt::AscendingOrder);
 
-    // activated() fails on X11 and Qtopia at least
-#if defined Q_WS_QWS || defined Q_WS_X11
-    disconnect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
-    connect(this, SIGNAL(doubleClicked(QModelIndex)), SLOT(joinChannel(QModelIndex)));
-#else
+#if defined Q_OS_MACOS || defined Q_OS_WIN
     // afaik this is better on Mac and Windows
     disconnect(this, SIGNAL(activated(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
     connect(this, SIGNAL(activated(QModelIndex)), SLOT(joinChannel(QModelIndex)));
+#else
+    disconnect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
+    connect(this, SIGNAL(doubleClicked(QModelIndex)), SLOT(joinChannel(QModelIndex)));
 #endif
 }
 
@@ -104,7 +106,7 @@ void BufferView::setModel(QAbstractItemModel *model)
 {
     delete selectionModel();
 
-    QTreeView::setModel(model);
+    TreeViewTouch::setModel(model);
     init();
     // remove old Actions
     QList<QAction *> oldactions = header()->actions();
@@ -129,6 +131,11 @@ void BufferView::setModel(QAbstractItemModel *model)
     }
 
     connect(model, SIGNAL(layoutChanged()), this, SLOT(on_layoutChanged()));
+
+    // Make sure collapsation is correct after setting a model
+    // This might not be needed here, only in BufferView::setFilteredModel().  If issues arise, just
+    // move down to setFilteredModel (which calls this function).
+    setExpandedState();
 }
 
 
@@ -211,16 +218,6 @@ void BufferView::joinChannel(const QModelIndex &index)
 }
 
 
-void BufferView::keyPressEvent(QKeyEvent *event)
-{
-    if (event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete) {
-        event->accept();
-        removeSelectedBuffers();
-    }
-    QTreeView::keyPressEvent(event);
-}
-
-
 void BufferView::dropEvent(QDropEvent *event)
 {
     QModelIndex index = indexAt(event->pos());
@@ -229,27 +226,44 @@ void BufferView::dropEvent(QDropEvent *event)
     QPoint cursorPos = event->pos();
 
     // check if we're really _on_ the item and not indicating a move to just above or below the item
+    // Magic margin number for this is from QAbstractItemViewPrivate::position()
     const int margin = 2;
     if (cursorPos.y() - indexRect.top() < margin
         || indexRect.bottom() - cursorPos.y() < margin)
-        return QTreeView::dropEvent(event);
+        return TreeViewTouch::dropEvent(event);
 
+    // If more than one buffer was being dragged, treat this as a rearrangement instead of a merge request
     QList<QPair<NetworkId, BufferId> > bufferList = Client::networkModel()->mimeDataToBufferList(event->mimeData());
     if (bufferList.count() != 1)
-        return QTreeView::dropEvent(event);
+        return TreeViewTouch::dropEvent(event);
 
+    // Get the Buffer ID of the buffer that was being dragged
     BufferId bufferId2 = bufferList[0].second;
 
-    if (index.data(NetworkModel::ItemTypeRole) != NetworkModel::BufferItemType)
-        return QTreeView::dropEvent(event);
-
-    if (index.data(NetworkModel::BufferTypeRole) != BufferInfo::QueryBuffer)
-        return QTreeView::dropEvent(event);
-
+    // Get the Buffer ID of the target buffer
     BufferId bufferId1 = index.data(NetworkModel::BufferIdRole).value<BufferId>();
-    if (bufferId1 == bufferId2)
-        return QTreeView::dropEvent(event);
 
+    // If the source and target are the same buffer, this was an aborted rearrangement
+    if (bufferId1 == bufferId2)
+        return TreeViewTouch::dropEvent(event);
+
+    // Get index of buffer that was being dragged
+    QModelIndex index2 = Client::networkModel()->bufferIndex(bufferId2);
+
+    // If the buffer being dragged is a channel and we're still joined to it, treat this as a rearrangement
+    // This prevents us from being joined to a channel with no associated UI elements
+    if (index2.data(NetworkModel::BufferTypeRole) == BufferInfo::ChannelBuffer && index2.data(NetworkModel::ItemActiveRole) == true)
+        return TreeViewTouch::dropEvent(event);
+
+    //If the source buffer is not mergeable(AKA not a Channel and not a Query), try rearranging instead
+    if (index2.data(NetworkModel::BufferTypeRole) != BufferInfo::ChannelBuffer && index2.data(NetworkModel::BufferTypeRole) != BufferInfo::QueryBuffer)
+        return TreeViewTouch::dropEvent(event);
+
+    // If the target buffer is not mergeable(AKA not a Channel and not a Query), try rearranging instead
+    if (index.data(NetworkModel::BufferTypeRole) != BufferInfo::ChannelBuffer && index.data(NetworkModel::BufferTypeRole) != BufferInfo::QueryBuffer)
+        return TreeViewTouch::dropEvent(event);
+
+    // Confirm that the user really wants to merge the buffers before doing so
     int res = QMessageBox::question(0, tr("Merge buffers permanently?"),
         tr("Do you want to merge the buffer \"%1\" permanently into buffer \"%2\"?\n This cannot be reversed!").arg(Client::networkModel()->bufferName(bufferId2)).arg(Client::networkModel()->bufferName(bufferId1)),
         QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
@@ -288,7 +302,7 @@ void BufferView::removeSelectedBuffers(bool permanently)
 
 void BufferView::rowsInserted(const QModelIndex &parent, int start, int end)
 {
-    QTreeView::rowsInserted(parent, start, end);
+    TreeViewTouch::rowsInserted(parent, start, end);
 
     // ensure that newly inserted network nodes are expanded per default
     if (parent.data(NetworkModel::ItemTypeRole) != NetworkModel::NetworkItemType)
@@ -312,7 +326,19 @@ void BufferView::on_configChanged()
 {
     Q_ASSERT(model());
 
-    // expand all active networks... collapse inactive ones... unless manually changed
+    // Expand/collapse as needed
+    setExpandedState();
+
+    if (config()) {
+        // update selection to current one
+        Client::bufferModel()->synchronizeView(this);
+    }
+}
+
+
+void BufferView::setExpandedState()
+{
+    // Expand all active networks, collapse inactive ones... unless manually changed
     QModelIndex networkIdx;
     NetworkId networkId;
     for (int row = 0; row < model()->rowCount(); row++) {
@@ -326,11 +352,6 @@ void BufferView::on_configChanged()
 
         setExpandedState(networkIdx);
     }
-
-    if (config()) {
-        // update selection to current one
-        Client::bufferModel()->synchronizeView(this);
-    }
 }
 
 
@@ -376,11 +397,11 @@ void BufferView::setExpandedState(const QModelIndex &networkIdx)
 #if QT_VERSION < 0x050000
 void BufferView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
 {
-    QTreeView::dataChanged(topLeft, bottomRight);
+    TreeViewTouch::dataChanged(topLeft, bottomRight);
 #else
 void BufferView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles)
 {
-    QTreeView::dataChanged(topLeft, bottomRight, roles);
+    TreeViewTouch::dataChanged(topLeft, bottomRight, roles);
 #endif
 
     // determine how many items have been changed and if any of them is a networkitem
@@ -480,6 +501,8 @@ void BufferView::changeBuffer(Direction direction)
     QModelIndex currentIndex = selectionModel()->currentIndex();
     QModelIndex resultingIndex;
 
+    QModelIndex lastNetIndex = model()->index(model()->rowCount() - 1, 0, QModelIndex());
+
     if (currentIndex.parent().isValid()) {
         //If we are a child node just switch among siblings unless it's the first/last child
         resultingIndex = currentIndex.sibling(currentIndex.row() + direction, 0);
@@ -496,6 +519,8 @@ void BufferView::changeBuffer(Direction direction)
         //If we have a toplevel node, try and get an adjacent child
         if (direction == Backward) {
             QModelIndex newParent = currentIndex.sibling(currentIndex.row() - 1, 0);
+            if (currentIndex.row() == 0)
+                newParent = lastNetIndex;
             if (model()->hasChildren(newParent))
                 resultingIndex = newParent.child(model()->rowCount(newParent) - 1, 0);
             else
@@ -509,18 +534,46 @@ void BufferView::changeBuffer(Direction direction)
         }
     }
 
-    if (!resultingIndex.isValid())
-        return;
+    if (!resultingIndex.isValid()) {
+        if (direction == Forward)
+            resultingIndex = model()->index(0, 0, QModelIndex());
+        else
+            resultingIndex = lastNetIndex.child(model()->rowCount(lastNetIndex) - 1, 0);
+    }
 
     selectionModel()->setCurrentIndex(resultingIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
     selectionModel()->select(resultingIndex, QItemSelectionModel::ClearAndSelect);
 }
 
+void BufferView::selectFirstBuffer()
+{
+    int networksCount = model()->rowCount(QModelIndex());
+    if (networksCount == 0) {
+        return;
+    }
+
+    QModelIndex bufferIndex;
+    for (int row = 0; row < networksCount; row++) {
+        QModelIndex networkIndex = model()->index(row, 0, QModelIndex());
+        int childCount = model()->rowCount(networkIndex);
+        if (childCount > 0) {
+            bufferIndex = model()->index(0, 0, networkIndex);
+            break;
+        }
+    }
+
+    if (!bufferIndex.isValid()) {
+        return;
+    }
+
+    selectionModel()->setCurrentIndex(bufferIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+    selectionModel()->select(bufferIndex, QItemSelectionModel::ClearAndSelect);
+}
 
 void BufferView::wheelEvent(QWheelEvent *event)
 {
     if (ItemViewSettings().mouseWheelChangesBuffer() == (bool)(event->modifiers() & Qt::AltModifier))
-        return QTreeView::wheelEvent(event);
+        return TreeViewTouch::wheelEvent(event);
 
     int rowDelta = (event->delta() > 0) ? -1 : 1;
     changeBuffer((Direction)rowDelta);
@@ -538,23 +591,26 @@ void BufferView::hideCurrentBuffer()
     //The check above means we won't be looking at a network, which should always be the first row, so we can just go backwards.
     changeBuffer(Backward);
 
-    /*if(removedRows.contains(bufferId))
-      continue;
-
-    removedRows << bufferId;*/
-    /*if(permanently)
-      config()->requestRemoveBufferPermanently(bufferId);
-    else*/
     config()->requestRemoveBuffer(bufferId);
 }
 
+void BufferView::filterTextChanged(const QString& filterString)
+{
+    BufferViewFilter *filter = qobject_cast<BufferViewFilter *>(model());
+    if (!filter) {
+        return;
+    }
+    filter->setFilterString(filterString);
+    on_configChanged(); // make sure collapsation is correct
+}
+
 
 QSize BufferView::sizeHint() const
 {
-    return QTreeView::sizeHint();
+    return TreeViewTouch::sizeHint();
 
     if (!model())
-        return QTreeView::sizeHint();
+        return TreeViewTouch::sizeHint();
 
     if (model()->rowCount() == 0)
         return QSize(120, 50);
@@ -568,8 +624,63 @@ QSize BufferView::sizeHint() const
 }
 
 
+void BufferView::changeHighlight(BufferView::Direction direction)
+{
+    // If for some weird reason we get a new delegate
+    BufferViewDelegate *delegate = qobject_cast<BufferViewDelegate*>(itemDelegate(_currentHighlight));
+    if (delegate) {
+        delegate->currentHighlight = QModelIndex();
+    }
+
+    QModelIndex newIndex = _currentHighlight;
+    if (!newIndex.isValid()) {
+        newIndex = model()->index(0, 0);
+    }
+
+    if (direction == Backward) {
+        newIndex = indexBelow(newIndex);
+    } else {
+        newIndex = indexAbove(newIndex);
+    }
+
+    if (!newIndex.isValid()) {
+        return;
+    }
+
+    _currentHighlight = newIndex;
+
+    delegate = qobject_cast<BufferViewDelegate*>(itemDelegate(_currentHighlight));
+    if (delegate) {
+        delegate->currentHighlight = _currentHighlight;
+    }
+    viewport()->update();
+}
+
+void BufferView::selectHighlighted()
+{
+    if (_currentHighlight.isValid()) {
+        selectionModel()->setCurrentIndex(_currentHighlight, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
+        selectionModel()->select(_currentHighlight, QItemSelectionModel::ClearAndSelect);
+    } else {
+        selectFirstBuffer();
+    }
+
+    clearHighlight();
+}
+
+void BufferView::clearHighlight()
+{
+    // If for some weird reason we get a new delegate
+    BufferViewDelegate *delegate = qobject_cast<BufferViewDelegate*>(itemDelegate(_currentHighlight));
+    if (delegate) {
+        delegate->currentHighlight = QModelIndex();
+    }
+    _currentHighlight = QModelIndex();
+    viewport()->update();
+}
+
 // ****************************************
-//  BufferViewDelgate
+//  BufferViewDelegate
 // ****************************************
 class ColorsChangedEvent : public QEvent
 {
@@ -605,7 +716,11 @@ bool BufferViewDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, c
     if (!value.isValid())
         return QStyledItemDelegate::editorEvent(event, model, option, index);
 
+#if QT_VERSION < 0x050000
     QStyleOptionViewItemV4 viewOpt(option);
+#else
+    QStyleOptionViewItem viewOpt(option);
+#endif
     initStyleOption(&viewOpt, index);
 
     QRect checkRect = viewOpt.widget->style()->subElementRect(QStyle::SE_ItemViewItemCheckIndicator, &viewOpt, viewOpt.widget);
@@ -631,6 +746,9 @@ bool BufferViewDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, c
 // ==============================
 BufferViewDock::BufferViewDock(BufferViewConfig *config, QWidget *parent)
     : QDockWidget(parent),
+    _childWidget(0),
+    _widget(new QWidget(parent)),
+    _filterEdit(new QLineEdit(parent)),
     _active(false),
     _title(config->bufferViewName())
 {
@@ -638,9 +756,32 @@ BufferViewDock::BufferViewDock(BufferViewConfig *config, QWidget *parent)
     toggleViewAction()->setData(config->bufferViewId());
     setAllowedAreas(Qt::RightDockWidgetArea|Qt::LeftDockWidgetArea);
     connect(config, SIGNAL(bufferViewNameSet(const QString &)), this, SLOT(bufferViewRenamed(const QString &)));
+    connect(config, SIGNAL(configChanged()), SLOT(configChanged()));
     updateTitle();
+
+    _widget->setLayout(new QVBoxLayout);
+    _widget->layout()->setSpacing(0);
+    _widget->layout()->setContentsMargins(0, 0, 0, 0);
+
+    // We need to potentially hide it early, so it doesn't flicker
+    _filterEdit->setVisible(config->showSearch());
+    _filterEdit->setFocusPolicy(Qt::ClickFocus);
+    _filterEdit->installEventFilter(this);
+    _filterEdit->setPlaceholderText(tr("Search..."));
+    connect(_filterEdit, SIGNAL(returnPressed()), SLOT(onFilterReturnPressed()));
+
+    _widget->layout()->addWidget(_filterEdit);
+    QDockWidget::setWidget(_widget);
 }
 
+void BufferViewDock::setLocked(bool locked) {
+    if (locked) {
+        setFeatures(0);
+    }
+    else {
+        setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    }
+}
 
 void BufferViewDock::updateTitle()
 {
@@ -650,17 +791,95 @@ void BufferViewDock::updateTitle()
     setWindowTitle(title);
 }
 
+void BufferViewDock::configChanged()
+{
+    if (_filterEdit->isVisible() != config()->showSearch()) {
+        _filterEdit->setVisible(config()->showSearch());
+        _filterEdit->clear();
+    }
+}
+
+void BufferViewDock::onFilterReturnPressed()
+{
+    if (_oldFocusItem) {
+        _oldFocusItem->setFocus();
+        _oldFocusItem = 0;
+    }
+
+    if (!config()->showSearch()) {
+        _filterEdit->setVisible(false);
+    }
+
+    BufferView *view = bufferView();
+    if (!view) {
+        return;
+    }
+
+    if (!_filterEdit->text().isEmpty()) {
+        view->selectHighlighted();
+        _filterEdit->clear();
+    } else {
+        view->clearHighlight();
+    }
+}
 
 void BufferViewDock::setActive(bool active)
 {
     if (active != isActive()) {
         _active = active;
         updateTitle();
-        if (active)
+        if (active) {
             raise();  // for tabbed docks
+        }
     }
 }
 
+bool BufferViewDock::eventFilter(QObject *object, QEvent *event)
+{
+   if (object != _filterEdit)  {
+       return false;
+   }
+
+   if (event->type() == QEvent::FocusOut) {
+       if (!config()->showSearch() && _filterEdit->text().isEmpty()) {
+           _filterEdit->setVisible(false);
+           return true;
+       }
+   } else if (event->type() == QEvent::KeyRelease) {
+       QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+
+       BufferView *view = bufferView();
+       if (!view) {
+           return false;
+       }
+
+       switch (keyEvent->key()) {
+       case Qt::Key_Escape: {
+           _filterEdit->clear();
+
+           if (!_oldFocusItem) {
+               return false;
+           }
+
+           _oldFocusItem->setFocus();
+           _oldFocusItem = nullptr;
+           return true;
+       }
+       case Qt::Key_Down:
+           view->changeHighlight(BufferView::Backward);
+           return true;
+       case Qt::Key_Up:
+           view->changeHighlight(BufferView::Forward);
+           return true;
+       default:
+           break;
+       }
+
+       return false;
+   }
+
+   return false;
+}
 
 void BufferViewDock::bufferViewRenamed(const QString &newName)
 {
@@ -691,3 +910,32 @@ BufferViewConfig *BufferViewDock::config() const
     else
         return view->config();
 }
+
+void BufferViewDock::setWidget(QWidget *newWidget)
+{
+    _widget->layout()->addWidget(newWidget);
+    _childWidget = newWidget;
+
+    connect(_filterEdit, SIGNAL(textChanged(QString)), bufferView(), SLOT(filterTextChanged(QString)));
+}
+
+void BufferViewDock::activateFilter()
+{
+    if (!_filterEdit->isVisible()) {
+        _filterEdit->setVisible(true);
+    }
+
+    _oldFocusItem = qApp->focusWidget();
+
+    _filterEdit->setFocus();
+}
+
+
+void BufferViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    QStyleOptionViewItem newOption = option;
+    if (index == currentHighlight) {
+        newOption.state |= QStyle::State_HasFocus;
+    }
+    QStyledItemDelegate::paint(painter, newOption, index);
+}
