@@ -19,7 +19,6 @@
  ***************************************************************************/
 
 #include <QApplication>
-#include <QMenu>
 #include <QMessageBox>
 #include <QScrollBar>
 
@@ -51,6 +50,11 @@ MultiLineEdit::MultiLineEdit(QWidget *parent)
     enableFindReplace(false);
 #endif
 
+#if defined HAVE_SONNET && !defined HAVE_KDE
+    _spellCheckDecorator = new Sonnet::SpellCheckDecorator(this);
+    highlighter()->setActive(highlighter()->checkerEnabledByDefault());
+#endif
+
     setMode(SingleLine);
     setLineWrapEnabled(false);
     reset();
@@ -83,15 +87,39 @@ MultiLineEdit::~MultiLineEdit()
 {
 }
 
-#if defined HAVE_KF5 || defined HAVE_KDE4
-void MultiLineEdit::createHighlighter()
+
+#if defined HAVE_SONNET && !defined HAVE_KDE
+Sonnet::Highlighter *MultiLineEdit::highlighter() const
 {
-    KTextEdit::createHighlighter();
-    if (highlighter())
-        highlighter()->setAutomatic(false);
+    return _spellCheckDecorator->highlighter();
 }
+
+
+void MultiLineEdit::setSpellCheckEnabled(bool enabled)
+{
+    highlighter()->setActive(enabled);
+    if (enabled) {
+        highlighter()->slotRehighlight();
+    }
+}
+
+void MultiLineEdit::contextMenuEvent(QContextMenuEvent *event)
+{
+    QMenu *menu = createStandardContextMenu();
+    menu->addSeparator();
+
+    auto action = menu->addAction(tr("Auto Spell Check"));
+    action->setCheckable(true);
+    action->setChecked(highlighter()->isActive());
+    connect(action, SIGNAL(toggled(bool)), this, SLOT(setSpellCheckEnabled(bool)));
+
+    menu->exec(event->globalPos());
+    delete menu;
+}
+
 #endif
 
+
 void MultiLineEdit::setCustomFont(const QFont &font)
 {
     setFont(font);
@@ -222,16 +250,6 @@ void MultiLineEdit::setEmacsMode(bool enable)
 }
 
 
-void MultiLineEdit::setSpellCheckEnabled(bool enable)
-{
-#ifdef HAVE_KDE
-    setCheckSpellingEnabled(enable);
-#else
-    Q_UNUSED(enable)
-#endif
-}
-
-
 void MultiLineEdit::setPasteProtectionEnabled(bool enable, QWidget *)
 {
     _pasteProtectionEnabled = enable;
