@@ -45,8 +45,10 @@ public:
         NetworkMask = 0x0f,
         NetworkConnect = 0x01,
         NetworkDisconnect = 0x02,
-        NetworkConnectAll = 0x03,
-        NetworkDisconnectAll = 0x04,
+        NetworkConnectAllWithDropdown = 0x03,
+        NetworkDisconnectAllWithDropdown = 0x04,
+        NetworkConnectAll = 0x05,
+        NetworkDisconnectAll = 0x06,
 
         // Buffer actions
         BufferMask = 0xf0,
@@ -73,6 +75,7 @@ public:
         JoinChannel = 0x1000,
         ShowChannelList = 0x2000,
         ShowIgnoreList = 0x3000,
+        ShowNetworkConfig = 0x4000,
 
         // Nick actions
         NickMask = 0xff0000,
@@ -151,7 +154,17 @@ protected slots:
     virtual void actionTriggered(QAction *);
 
 signals:
-    void showChannelList(NetworkId);
+    /**
+     * Request to show the channel list dialog for the network, optionally searching by channel name
+     *
+     * @see MainWin::showChannelList()
+     *
+     * @param networkId        Network ID for associated network
+     * @param channelFilters   Partial channel name to search for, or empty to show all
+     * @param listImmediately  If true, immediately list channels, otherwise just show dialog
+     */
+    void showChannelList(NetworkId, const QString &, bool);
+    void showNetworkConfig(NetworkId);
     void showIgnoreList(QString);
 
 protected:
